// Spatial Index Library
//
// Copyright (C) 2002 Navel Ltd.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    lifeifei@cs.bu.edu, mhadji@gmail.com

// NOTE: Please read README.txt before browsing this code.

// include library header file.
#include <SpatialIndex.h>

using namespace SpatialIndex;
using namespace std;

#define INSERT 1
#define DELETE 0
#define QUERY 2

// example of a Visitor pattern.
// findes the index and leaf IO for answering the query and prints
// the resulting data IDs to stdout.
class MyVisitor : public IVisitor
{
public:
	unsigned long m_indexIO;
	unsigned long m_leafIO;
	vector<byte* > vos;	
	vector<long > digestSize;
	vector<long> digPosition;
	vector<long> visitedNodes;

public:
	MyVisitor() : m_indexIO(0), m_leafIO(0) 
	{	
	}

	void visitNode(const INode& n)
	{
		if (n.isLeaf()) m_leafIO++;
		else m_indexIO++;
		visitedNodes.push_back(n.getIdentifier());

	}

	void visitData(const IData& d)
	{
		IShape* pS;
		d.getShape(&pS);
			// do something.
		Point key;
		pS->getCenter(key);	
		delete pS;

		// data should be an array of characters representing a Region as a string.
		//byte* pData = 0;
		//unsigned long cLen = 0;
		//d.getData(cLen, &pData);
			// do something.
			//string s = reinterpret_cast<char*>(pData);
			//cout << s << endl;
		//delete[] pData;

		//cout << "Query results: pid key "<<endl;
		//cout << d.getIdentifier() << " "<< key.m_pCoords[0]<<endl;
			// the ID of this data entry is an answer to the query. I will just print it to stdout.
	}

	void visitData(std::vector<const IData*>& v)
	{
		cout << v[0]->getIdentifier() << " " << v[1]->getIdentifier() << endl;
	}
	
	void updateVOs(byte** digest, long num, long pos) 
	{
		digestSize.insert(digestSize.begin(),num);
		digPosition.insert(digPosition.begin(),pos);
		for (long i=num-1;i>=0;i--)
		{
			byte* dig=(byte*)malloc(DIGEST_LENGTH*sizeof(byte));
			memcpy(dig, digest[i], DIGEST_LENGTH);
			vos.insert(vos.begin(),dig);	
		}		
		delete[] digest;
	}

	void updateVOs(byte** digest, long num, long* pos, long posnum) 
	{
		digestSize.insert(digestSize.begin(),num);
		for (long i=num-1;i>=0;i--)
		{
			byte* dig=(byte*)malloc(DIGEST_LENGTH*sizeof(byte));
			memcpy(dig, digest[i], DIGEST_LENGTH);
			vos.insert(vos.begin(),dig);	
		}		
		for (long i=posnum-1;i>=0;i--)
		{
			digPosition.insert(digPosition.begin(),pos[i]);
		}		

		delete[] pos;
		delete[] digest;
	}

};

void verify(vector<long> digestSize, vector<byte*> vos, vector<long> digPosition, long key, long fanout) 
{
	long size=digestSize.size();
	byte* kd=(byte*)malloc(SHA_DIGEST_LENGTH*sizeof(key));
	byte* input=(unsigned char*)&key;			
	SpatialIndex::sha1(input, kd, sizeof(long));		
	byte* lastDigest=kd;
	byte* digests;
	long voSize=vos.size();

	EVP_MD_CTX tc;
	for (long i=0; i< size; i++)
	{
		long s=digestSize.front();
		digestSize.erase(digestSize.begin());

		long pos=digPosition.front();
		digPosition.erase(digPosition.begin());
		bool meetPos=false;
		long k=0;
		long operation=0;
		EVP_MD_CTX_init(&tc);
		EVP_DigestInit_ex(&tc,EVP_sha1(), NULL);
		digests=(byte*)malloc(DIGEST_LENGTH*sizeof(byte));
		cout<<"s: "<<s<<endl;
		cout<<"pos: "<<pos<<endl;
		for (long j=0; j<s; j++)
		{
			k=operation%fanout;
			if(k==0 && operation!=0)
			{	
				EVP_DigestFinal_ex(&tc,digests,NULL);
				EVP_MD_CTX_cleanup(&tc);
				lastDigest=digests;
				
				EVP_MD_CTX_init(&tc);
				EVP_DigestInit_ex(&tc,EVP_sha1(), NULL);
				digests=(byte*)malloc(DIGEST_LENGTH*sizeof(byte));

				pos=digPosition.front();
				digPosition.erase(digPosition.begin());
				meetPos=false;
				cout<<"pos: "<<pos<<endl;
			}
			if (k==pos && meetPos==false)
			{	
				char* d= SpatialIndex::printDigest(lastDigest, DIGEST_LENGTH);			
				for (unsigned long i=0; i< DIGEST_LENGTH*2; i++)
					cout << d[i];
				cout<<"     (computed digest) "<<endl;
				delete[] d;
				EVP_DigestUpdate(&tc,lastDigest,DIGEST_LENGTH);
				delete[] lastDigest;
				meetPos=true;
				j--;
			}
			else
			{
				byte* dig=vos.front();
				vos.erase(vos.begin());
				char* d= SpatialIndex::printDigest(dig, DIGEST_LENGTH);
				for (unsigned long i=0; i< DIGEST_LENGTH*2; i++)
					cout << d[i];
				cout<<endl;
				delete[] d;
				EVP_DigestUpdate(&tc,dig,DIGEST_LENGTH);
				delete[] dig;
			}
			operation++;
		}	
		k=operation%fanout;
		if (k==pos && meetPos==false)
		{
			char* d= SpatialIndex::printDigest(lastDigest, DIGEST_LENGTH);			
			for (unsigned long i=0; i< DIGEST_LENGTH*2; i++)
				cout << d[i];
			cout<<"     (computed digest) "<<endl;
			delete[] d;
			EVP_DigestUpdate(&tc,lastDigest,DIGEST_LENGTH);
			delete[] lastDigest;
			meetPos=true;
		}
		EVP_DigestFinal_ex(&tc,digests,NULL);
		EVP_MD_CTX_cleanup(&tc);
		lastDigest=digests;
	}
	cout<<"query verify digest:"<<endl;
	char* d= SpatialIndex::printDigest(lastDigest, DIGEST_LENGTH);		
	for (unsigned long i=0; i< DIGEST_LENGTH*2; i++)
		cout << d[i];
	cout<<endl;
	cout<<"VOs size and total length:"<<voSize<<" "<<voSize*DIGEST_LENGTH<<endl;
	cout<<endl;
	delete[] d;
	delete[] lastDigest;
}

int main(int argc, char** argv)
{
	try
	{
		if (argc != 5)
		{
			cerr << "Usage: " << argv[0] << " query_file tree_file query_type [pointsearch | rangequery] embedding_fanout." << endl;
			return -1;
		}

		int queryType = 0;

		if (strcmp(argv[3], "pointsearch") == 0) queryType = 0;
		else if (strcmp(argv[3], "rangequery") == 0) queryType = 1;
		else
		{
			cerr << "Unknown query type." << endl;
			return -1;
		}

		ifstream fin(argv[1]);
		if (! fin)
		{
			cerr << "Cannot open query file " << argv[1] << "." << endl;
			return -1;
		}
		
		long embeddingFanout=atoi(argv[4]);

		string baseName = argv[2];
		IStorageManager* diskfile = StorageManager::loadDiskStorageManager(baseName);
			// this will try to locate and open an already existing storage manager.

		StorageManager::IBuffer* file = StorageManager::createNewRandomEvictionsBuffer(*diskfile, 10, false);
			// applies a main memory random buffer on top of the persistent storage manager
			// (LRU buffer, etc can be created the same way).

		// If we need to open an existing tree stored in the storage manager, we only
		// have to specify the index identifier as follows

		ISpatialIndex* tree = BTree::loadBTree(*file, 1);
		
		unsigned long count = 0;
		unsigned long indexIO = 0;
		unsigned long leafIO = 0;
		unsigned long id, op;
		double x[1], y[1];
		Tools::ResourceUsage ru;

		ru.start();

		while (fin)
		{
			fin >> op >> id >> x[0] >> y[0];
			if (! fin.good()) continue; // skip newlines, etc.

			if (op == QUERY)
			{

				MyVisitor vis;

				if (queryType == 0)
				{
					Point p = Point(x, 1);
					tree->pointLocationQuery(p, vis);
					verify(vis.digestSize, vis.vos, vis.digPosition, (long)x[0], embeddingFanout);

				}
				else if (queryType == 1)
				{
					LineSegment r = LineSegment(x, y, 1);
					//tree->containsWhatQuery(r, vis);
				}
				//indexIO += vis.m_indexIO;
				//leafIO += vis.m_leafIO;
					// example of the Visitor pattern usage, for calculating how many nodes
					// were visited.
				if (queryType==1)
				{
					MyVisitor vis1;
					Point p1 = Point(x, 1);
					tree->pointLocationQuery(p1, vis1);

					MyVisitor vis2;
					Point p2 = Point(y, 1);
					tree->pointLocationQuery(p2, vis2);
					int common=0;
					for (int i=0;i<vis2.visitedNodes.size();i++)
					{
						for (int j=0;j<vis1.visitedNodes.size();j++)
						{
							if (vis2.visitedNodes[i]==vis1.visitedNodes[j])
							{
								common++;
								break;
							}
						}
					}
					indexIO+=(vis2.visitedNodes.size()-common);	
				}

			}
			else
			{
				cerr << "This is not a query operation." << endl;
			}

			if ((count % 1000) == 0 && count!=0) cerr << count << endl;
			count++;
		}

		ru.stop();

		cerr << "Operations: " << count << endl;
		cerr << *tree;
		cerr << "Index I/O: " << indexIO << endl;
		cerr << "Leaf I/O: " << leafIO << endl;
		cerr << "Buffer hits: " << file->getHits() << endl;
		cerr << "Seconds: " << ru.getTotalTime() << endl;

		delete tree;
		delete file;
		delete diskfile;
			// delete the buffer first, then the storage manager
			// (otherwise the the buffer will fail writting the dirty entries).
	}
	catch (Tools::Exception& e)
	{
		cerr << "******ERROR******" << endl;
		std::string s = e.what();
		cerr << s << endl;
		return -1;
	}
	catch (...)
	{
		cerr << "******ERROR******" << endl;
		cerr << "other exception" << endl;
		return -1;
	}

	return 0;
}
