// Spatial Index Library
//
// Copyright (C) 2002 Navel Ltd.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    lifeifei@cs.bu.edu, mhadji@gmail.com

// NOTE: Please read README.txt before browsing this code.

// include library header file.
#include <SpatialIndex.h>

using namespace SpatialIndex;
using namespace std;

#define INSERT 1
#define DELETE 0
#define QUERY 2

int main(int argc, char** argv)
{
	try
	{
		if (argc != 5)
		{
			cerr << "Usage: " << argv[0] << " input_file tree_file capacity query_type [pointsearch | rangequery]." << endl;
			return -1;
		}

		int queryType = 0;

		if (strcmp(argv[4], "pointsearch") == 0) queryType = 0;
		else if (strcmp(argv[4], "rangequery") == 0) queryType = 1;
		else
		{
			cerr << "Unknown query type." << endl;
			return -1;
		}

		ifstream fin(argv[1]);
		if (! fin)
		{
			cerr << "Cannot open data file " << argv[1] << "." << endl;
			return -1;
		}

		char* treefile = argv[2];
		RBTree* tree= new RBTree();
		//ISpatialIndex* tree = BTree::createNewBTree(*file, 0.7, atoi(argv[3]), atoi(argv[3]), 1, SpatialIndex::BTree::BV_PLUS, indexIdentifier);

		unsigned long count = 0;
		unsigned long id, op;
		long x;
		long y;
		Tools::ResourceUsage ru;
		std::vector<long> keys;
		ru.start();

		while (fin)
		{
			fin >> op >> id >> x >> y;
			if (! fin.good()) continue; // skip newlines, etc.

			if (op == INSERT)
			{
				keys.push_back(x);
			}
			else if (op == DELETE)
			{

			}
			else if (op == QUERY)
			{

				cout << "query "<<id<<" "<<x<<" "<<y<<endl;

				if (queryType == 0)
				{
				}
			}

			if (((count+1) % 1000) == 0 && count!=0)
				cerr << count+1 << endl;
			count++;
		}
		//std::sort(keys.begin(), keys.end());
		
		long size=keys.size();
		long* keyset= new long[size];
		long* pid = new long[size];
		int* nid=new int[size];
		byte** digest = new byte*[size];
		for (long i=0;i<size;i++)
		{
			keyset[i]=(long)keys.front();
			pid[i]=0;
			digest[i]=NULL;
			nid[i]=i;
			keys.erase(keys.begin());
		}
		
		tree->bulkloadTree(keyset, pid, nid,digest, size);
		tree->printTree(tree->m_root,40);
		tree->showTree();
		cout<<"hash computed: "<<tree->m_hashComputed <<endl;
		cout<<"computation cost: "<<tree->m_cputime <<endl;
		tree->storeToFile(treefile);
		delete tree;
		tree=new RBTree();
		tree->loadFromFile(treefile);
		tree->printTree(tree->m_root,40);
		tree->showTree();
		delete tree;
			
		byte** vos;
		long voSize;
		long* pos;				
		long posSize;
		long searchKey=3;
		/*tree->constructVOs(searchKey, vos, voSize, &pos, posSize);
		cout<< "vo size: " << voSize <<endl;
		for (long i=0; i<voSize; i++)
		{
			char* d= SpatialIndex::printDigest(vos[i], DIGEST_LENGTH);		
			for (long j=0; j< DIGEST_LENGTH*2; j++)
				cout << d[j];
			cout<<endl;
		}
		delete[] pos;
		delete[] vos;*/

		ru.stop();

		cerr << "Operations: " << count << endl;
		cerr << "Seconds: " << ru.getTotalTime() << endl;
	

		//delete tree;
	}
	catch (Tools::Exception& e)
	{
		cerr << "******ERROR******" << endl;
		std::string s = e.what();
		cerr << s << endl;
		return -1;
	}
	catch (...)
	{
		cerr << "******ERROR******" << endl;
		cerr << "other exception" << endl;
		return -1;
	}

	return 0;
}
