// Spatial Index Library
//
#ifndef __spatialindex_nwaynode_h
#define __spatialindex_nwaynode_h


namespace SpatialIndex
{
	class NwayTree;
	
	class NwayNode : public Tools::ISerializable, public Tools::IObject
	{
	public:
		NwayNode();
		NwayNode(NwayTree* tree);
		NwayNode(NwayTree* tree, byte* const data);
		NwayNode::NwayNode(NwayTree* tree, long level, long identifier, long children, long* keys, NwayNode** childs, long* childsID, byte** digests);
		virtual ~NwayNode();

		//
		// IObject interface
		//
		virtual NwayNode* clone() throw (Tools::NotSupportedException);

		//
		// ISerializable interface
		//
		virtual unsigned long getByteArraySize();
		virtual unsigned long loadFromByteArray(byte* const data);
		virtual void storeToByteArray(unsigned long& len, byte** data);
	
		// Public methods	
		virtual void computeNodeDigests(byte* digest);
		virtual long getSlot(long key);
		virtual void showNode();

	public:
		long m_identifier;
		long m_fanout;
		long m_children;
		long m_level;
		long* m_pKey;
		NwayNode** m_pChild;
		long* m_pChildID;
		byte** m_pDigest;
		NwayTree* m_pTree;
		
		friend class NwayTree;

		friend std::ostream& operator<<(std::ostream& os, const NwayNode& r);
	}; // NwayNode 

	std::ostream& operator<<(std::ostream& os, const NwayNode& r);
}

#endif /* __spatialindex_nwaynode_h */
