// Spatial Index Library
//
// Copyright (C) 2002 Navel Ltd.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    lifeifei@cs.bu.edu, mhadji@gmail.com

#include "../spatialindex/SpatialIndexImpl.h"

#include "BTree.h"
#include "Node.h"
#include "Leaf.h"

#include "Index.h"

using namespace std;
using namespace SpatialIndex::BTree;

Index::~Index()
{
}

Index::Index(BTree* pTree, long id, unsigned long level) : Node(pTree, id, level, pTree->m_indexCapacity)
{
	m_minFanout=((long)(ceil((double(m_capacity+1))/2.0)))-1;
}

NodePtr Index::chooseSubtree(const Point& key, unsigned long level, std::stack<long>& pathBuffer)
{
	// should make sure to relinquish other PoolPointer lists that might be pointing to the
	// same leaf.
	return NodePtr(this, &(m_pTree->m_indexPool));
}

NodePtr Index::findLeaf(const Point& key, long id, std::stack<long>& pathBuffer)
{
	for (unsigned long cChild = 0; cChild < m_children; cChild++)
	{
		// should make sure to relinquish other PoolPointer lists that might be pointing to the
		// same leaf.
		if (m_pIdentifier[cChild] == id && key == *(m_ptrKey[cChild])) return NodePtr(this, &(m_pTree->m_indexPool));
	}

	return NodePtr();
}

void Index::showNode()
{
	cout<<"Index Node "<<m_identifier<<" with "<<m_children<<" entries"<<endl;
	cout<<"Capcity "<<m_capacity<<" MinFanout "<<m_minFanout<<endl;	
	cout<<"node="<<m_pFirstNode<<" ";
	if (m_totalDigestLength>0)
	{	
		cout<<"hashes=";
		unsigned long i=0;
		for (i=0; i<DIGEST_LENGTH; i++)
			printf("%02x",m_pFirstDigest[i]); 
	}
	for (long cChild=0;cChild<m_children;cChild++)
	{
		cout<<" key="<<(*(m_ptrKey[cChild])).m_pCoords[0]<<" node="<<m_pIdentifier[cChild]<<" ";
		if (m_totalDigestLength>0)
		{	
					cout<<"	hashes=";
					char* d= printDigest(m_pDigest[cChild], DIGEST_LENGTH);		
					for (unsigned long i=0; i< DIGEST_LENGTH*2; i++)
							cout << d[i];
					delete[] d;
		}			
	}
	cout<<endl;
	cout<<endl;
}
