// Spatial Index Library
//

#ifndef __spatialindex_nwaytree_h
#define __spatialindex_nwaytree_h

#include "NwayNode.h"

namespace SpatialIndex
{
	class NwayTree : public Tools::ISerializable, public Tools::IObject
	{
	public:
		NwayTree();
		NwayTree(long fanout);
		NwayTree(long fanout, byte* const data);
		virtual ~NwayTree();

		//
		// IObject interface
		//
		virtual NwayTree* clone() throw (Tools::NotSupportedException);

		//
		// ISerializable interface
		//
		virtual unsigned long getByteArraySize();
		virtual unsigned long loadFromByteArray(byte* const data);
		virtual void storeToByteArray(unsigned long& len, byte** data);

		//
		// Own public method
		//
		virtual void bulkloadTree(long* keys, long size, byte** indigests);
		virtual void writeNode(NwayNode* node, byte** buffer, unsigned long& dataLength);
		virtual void writeTree(byte** buffer, unsigned long& dataLength);
		virtual NwayNode* readNode(byte*& buffer);
		virtual void readTree(byte* buffer);
		virtual void constructVOs(long searchKey, byte**& voDigest, long& voSize, long** pos, long& posSize);
		virtual void showTree();
		virtual void storeToFile(char* file);
		virtual void loadFromFile(char* file);
		virtual void getLeafDigests(byte** digests);
	public:
		long m_fanout;
		NwayNode* m_root;
		long m_rootID;
		long m_nodeID;		
		byte* m_treeDigest;
		clock_t m_c0;
		clock_t m_c1;
		double m_cputime;
		long m_hashComputed;
	
		friend class NwayNode;
		
		friend std::ostream& operator<<(std::ostream& os, const NwayTree& ntree);
	}; //NwayTree 

	std::ostream& operator<<(std::ostream& os, const NwayTree& ntree);
}

#endif /*__spatialindex_nwaytree_h*/
