#include <SpatialIndex.h>

using namespace SpatialIndex;

//constructor
RBNode::RBNode(long key, COLOR color):
key(key),
color(color),
left(NULL),
right(NULL),
parent(NULL),
cDigest(NULL),
lDigest(NULL),
rDigest(NULL),
m_pID(-1),
m_child(0),
m_leftID(-1),
m_rightID(-1),
m_tree(NULL)
{}

RBNode::RBNode(long key):
key(key),
color(RED),
left(NULL),
right(NULL),
parent(NULL),
cDigest(NULL),
lDigest(NULL),
rDigest(NULL),
m_pID(-1),
m_child(0),
m_leftID(-1),
m_rightID(-1),
m_tree(NULL),
m_id(-1)
{}

RBNode::RBNode(RBTree* tree):
key(0),
color(RED),
left(NULL),
right(NULL),
parent(NULL),
cDigest(NULL),
lDigest(NULL),
rDigest(NULL),
m_pID(-1),
m_child(0),
m_leftID(-1),
m_rightID(-1),
m_tree(tree),
m_id(-1)
{}


RBNode::RBNode(RBTree* tree, long key, long pID, int id, byte* hash):
key(key),
color(RED),
left(NULL),
right(NULL),
parent(NULL),
cDigest(NULL),
lDigest(NULL),
rDigest(NULL),
m_pID(pID),
m_child(0),
m_leftID(-1),
m_rightID(-1),
m_tree(tree)
{
	cDigest=(byte*)malloc(DIGEST_LENGTH*sizeof(byte));
	if (hash==NULL)
	{
		m_tree->m_c0=clock();
		byte* input=(unsigned char*)&(key);
		sha1(input, cDigest,sizeof(long));	
		m_tree->m_c1=clock();
		m_tree->m_cputime+=(double)(m_tree->m_c1-m_tree->m_c0)/(double)(CLOCKS_PER_SEC);
		m_tree->m_hashComputed++;
	}
	else
		memcpy(cDigest, hash, DIGEST_LENGTH);
	if (id==-1)
		m_id=m_tree->m_id++;	
	else	
		m_id=id;
}


//destructor
RBNode::~RBNode()
{
	//std::cout <<"Destructor called " << key <<std::endl;
	if(left != NULL)
		left->parent = NULL;
	if(right != NULL)
		right->parent = NULL;
	if (cDigest !=NULL)
		delete cDigest;
	if (lDigest!=NULL)
		delete lDigest;
	if (rDigest!=NULL)
		delete rDigest;
	delete left;
	delete right;
}

unsigned long RBNode::getByteArraySize()
{
	unsigned long nodeLen= sizeof(int) + sizeof(long) + sizeof(int) + m_child*(sizeof(int)+DIGEST_LENGTH)+DIGEST_LENGTH;
	return nodeLen;
}

unsigned long RBNode::loadFromByteArray(byte* const data)
{
	byte* ptr = data;
	
	memcpy(&m_id, ptr, sizeof(int));
	ptr+=sizeof(int);

	memcpy(&key, ptr, sizeof(long));
	ptr+=sizeof(long);

	memcpy(&m_child, ptr, sizeof(int));
	ptr+=sizeof(int);


	for (int i=1; i<=m_child; i++)
	{
		if (i==1)
		{
			memcpy(&m_leftID, ptr, sizeof(int));
			ptr+=sizeof(int);
			lDigest=new byte[DIGEST_LENGTH];	
			memcpy(lDigest, ptr, DIGEST_LENGTH);
			ptr+=DIGEST_LENGTH;
		}
		if (i==2)
		{
			memcpy(&m_rightID, ptr, sizeof(int));
			ptr+=sizeof(int);
			rDigest=new byte[DIGEST_LENGTH];	
			memcpy(rDigest, ptr, DIGEST_LENGTH);
			ptr+=DIGEST_LENGTH;
		}
	}
	
	cDigest=new byte[DIGEST_LENGTH];	
	memcpy(cDigest, ptr, DIGEST_LENGTH);

	return getByteArraySize();
}

void RBNode::storeToByteArray(unsigned long& len, byte** data)
{
	len = getByteArraySize();
	*data = new byte[len];
	byte* ptr = *data;
	
	memcpy(ptr, &m_id, sizeof(int));
	ptr+=sizeof(int);
	
	memcpy(ptr, &key, sizeof(long));
	ptr+=sizeof(long);

	memcpy(ptr, &m_child, sizeof(int));
	ptr+=sizeof(int);

	for (long i=1; i<=m_child; i++)
	{
		if (i==1)
		{
			memcpy(ptr, &(m_leftID), sizeof(int));
			ptr+=sizeof(int);
			memcpy(ptr, lDigest, DIGEST_LENGTH);
			ptr+=DIGEST_LENGTH;
		}
		if (i==2)
		{
			memcpy(ptr, &(m_rightID), sizeof(int));
			ptr+=sizeof(int);
			memcpy(ptr, rDigest, DIGEST_LENGTH);
			ptr+=DIGEST_LENGTH;
		}
	}
	memcpy(ptr, cDigest, DIGEST_LENGTH);
	ptr+=DIGEST_LENGTH;
	assert(len==(ptr-*data));
}


void RBNode::showNode()
{
	unsigned long j;
	if (m_leftID!=-1)
	{	
		printf("child %d ", m_leftID);
		if (lDigest!=NULL)
		{
			j=0;
			for (j=0; j<DIGEST_LENGTH; j++)
				printf("%02x",lDigest[j]); 
		}
	}

	printf(" key %d id %d ", key,m_id);
	
	if (cDigest!=NULL)
	{
		j=0;
		for (j=0; j<DIGEST_LENGTH; j++)
			printf("%02x",cDigest[j]); 
	}
	
	if (m_rightID!=-1)
	{	
		printf(" child %d ", m_rightID);
		if (rDigest!=NULL)
		{	
			j=0;
			for (j=0; j<DIGEST_LENGTH; j++)
				printf("%02x",rDigest[j]);
		}
	}
	printf("\n");
}

void RBNode::computeNodeDigest(byte* output)
{
	int count=0;
	if (lDigest!=NULL)
		count++;
	if (cDigest!=NULL)
		count++;
	if (rDigest!=NULL)
		count++;		
	if (count>0)
	{
		byte* input = (byte*)malloc(count*DIGEST_LENGTH);
		byte* tmp=input;
		if (lDigest!=NULL)
		{
			memcpy(tmp, lDigest, DIGEST_LENGTH);
			tmp+=DIGEST_LENGTH;
		}
		if (cDigest!=NULL)
		{
			memcpy(tmp, cDigest, DIGEST_LENGTH);
			tmp+=DIGEST_LENGTH;
		}
		if (rDigest!=NULL)
		{
			memcpy(tmp, rDigest, DIGEST_LENGTH);
			tmp+=DIGEST_LENGTH;
		}
		if (count>1)
		{
			//node->m_tree->m_c0=clock();
			sha1(input, output, count*DIGEST_LENGTH);	
			//node->m_tree->m_c1=clock();
			//node->m_tree->m_cputime+=(double)(node->m_tree->m_c1-node->m_tree->m_c0)/(double)(CLOCKS_PER_SEC);
			//node->m_tree->m_hashComputed++;

		}
		else
			memcpy(output, input, DIGEST_LENGTH);
		delete input;
	}

}
